@@ -1,11 +1,10 @@
-from collections import namedtuple
-#!/usr/bin/env python
 # -*- coding: utf-8 -*-
 
 """
 Tests that don't hit the Google Music servers.
 """
 
+from collections import namedtuple
 import time
 
 from mock import MagicMock as Mock
@@ -19,6 +18,7 @@ import gmusicapi.session
 from gmusicapi.clients import Webclient, Musicmanager
 from gmusicapi.exceptions import AlreadyLoggedIn  # ,NotLoggedIn
 from gmusicapi.protocol.shared import authtypes
+from gmusicapi.protocol import mobileclient
 from gmusicapi.utils import utils
 
 
@@ -169,6 +169,13 @@ def authtypes_factory_args():
     assert_false(auth.xt)
 
 
+@test
+def mc_url_signing():
+    sig, _ = mobileclient.GetStreamUrl.get_signature("Tdr6kq3xznv5kdsphyojox6dtoq",
+                                                     "1373247112519")
+    assert_equal(sig, "gua1gInBdaVo7_dSwF9y0kodua0")
+
+
 ##
 # utils
 ##
